// last revised: 14:08 5/8/2010
// tested on fx3, fx3.5, fx3.6, ie6, ie7, ie8, chrome5, opera10, safari5, XHTML 1.0 Transitional

/**
 * This class is used to perform easy AJAX data request (HTTP).
 * This class include both GET and POST method.
 * This class support response data be HTML or XML.
 */
function ModuleAjax() {

	var WSObj = new Object();

	/**
	 * private array recvType, child element is either 'HTML' or 'XML'
	 * private array resultHandle, child element is function
	 * private array XMLHttpRequest, child element is AJAX HTTP request/connection handler
	 */
	WSObj.recvType = new Array();
	WSObj.resultHandle = new Array();
	WSObj.XMLHttpRequest = new Array();

	/**
	 * Private. Prepare a AJAX connection, no URL is involved so far
	 * @return Return the request handler.
	 */
	WSObj.createXMLHttpRequest = function() {
		var request;

		if (window.ActiveXObject) {
			try {
				request = new ActiveXObject('Microsoft.XMLHTTP');
			} catch (e) {
				request = false;
			}
		} else if (window.XMLHttpRequest) {
			request = new XMLHttpRequest();
		}

		if (!request) alert('對不起, 你現在使用的瀏覽器不支援 AJAX. Your browser do not support AJAX request.');

		return request;
	}

	/**
	 * Private. After we have got the response from AJAX request, we call this function to handle the rest(final) of the works.
	 * @param integer diffhash the unique identifier of each connection
	 * @return Nothing return.
	 */
	WSObj.processHandle = function(diffhash) {
		if (WSObj.XMLHttpRequest[diffhash].readyState == 4) {
			if (WSObj.XMLHttpRequest[diffhash].status == 200) {
				if (WSObj.recvType[diffhash] == 'HTML') {
					WSObj.resultHandle[diffhash](WSObj.XMLHttpRequest[diffhash].responseText);
				} else if (WSObj.recvType[diffhash] == 'XML') {
					WSObj.resultHandle[diffhash](WSObj.XMLHttpRequest[diffhash].responseXML.lastChild.firstChild.nodeValue);
				}
			} else {
				alert('AJAX: Page not found.');
			}
			WSObj.deleteRequest(diffhash);
		}
	}

	/**
	 * @param string recvType either 'HTML' or 'XML'
	 * @param string targetUrl the complete URL of the destination, should include protocal type, e.g. 'http://'
	 * @param function resultHandle after the AJAX request is completed, the obtained data will be passed into the resultHandle(obtained_data), e.g. function(content) { alert(content); }
	 * @return the identifier of the AJAX request
	 */
	WSObj.get = function(recvType, targetUrl, resultHandle) {
		var diffhash = (new Date()).getTime();

		WSObj.recvType[diffhash] = recvType == 'HTML' ? 'HTML' : 'XML';

		WSObj.XMLHttpRequest[diffhash] = WSObj.createXMLHttpRequest();
		WSObj.resultHandle[diffhash] = resultHandle;
		WSObj.XMLHttpRequest[diffhash].onreadystatechange = function () { WSObj.processHandle(diffhash) };

		WSObj.XMLHttpRequest[diffhash].open('GET', targetUrl + (targetUrl.match(/\?/) == '?' ? '&' : '?') + 'diffhash=' + diffhash, true);
		WSObj.XMLHttpRequest[diffhash].send(null);

		return diffhash;
	}

	/**
	 * @param string recvType either 'HTML' or 'XML'
	 * @param string targetUrl the complete URL of the destination, should include protocal type, e.g. 'http://'
	 * @param string sendString data that want to send by POST, no '?' at the beginning, use '&' instead of '&amp;', e.g. var1=1&var2=2&var3=3
	 * @param function resultHandle after the AJAX request is completed, the obtained data will be passed into the resultHandle(obtained_data), e.g. function(content) { alert(content); }
	 * @return the identifier of the AJAX request
	 */
	WSObj.post = function(recvType, targetUrl, sendString, resultHandle) {
		var diffhash = (new Date()).getTime();
		sendString = sendString + (sendString ? '&' : '') + 'diffhash=' + diffhash;

		WSObj.recvType[diffhash] = recvType == 'HTML' ? 'HTML' : 'XML';

		WSObj.XMLHttpRequest[diffhash] = WSObj.createXMLHttpRequest();
		WSObj.resultHandle[diffhash] = resultHandle;
		WSObj.XMLHttpRequest[diffhash].onreadystatechange = function () { WSObj.processHandle(diffhash) };

		WSObj.XMLHttpRequest[diffhash].open('POST', targetUrl, true);
		WSObj.XMLHttpRequest[diffhash].setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		WSObj.XMLHttpRequest[diffhash].send(sendString);

		return diffhash;
	}

	/**
	 * @return integer 1 if succeed, -1 if browser don't support abort action, -2 if that AJAX request not exist
	 */
	WSObj.abort = function(label) {
		if (typeof WSObj.XMLHttpRequest[label] != 'undefined' && WSObj.XMLHttpRequest[label] != null) {
			// IE 6 or below don't support abort
			if (typeof WSObj.XMLHttpRequest[label].abort != 'undefined' || WSObj.XMLHttpRequest[label].abort != null) {
				return -1;
			}
			WSObj.XMLHttpRequest[label].abort();
			WSObj.deleteRequest(label);
			return 1;
		}
		return -2;
	}

	WSObj.deleteRequest = function (label) {
		delete WSObj.recvType[label];
		delete WSObj.resultHandle[label];
		delete WSObj.XMLHttpRequest[label];
	}

	return WSObj;
}